///////////////////////////////////////////////////////////////////////////////
//	モジュール : Smap.controllers.gamepad.js, 2015-04-26, 西岡 芳晴 ( NISHIOKA Yoshiharu ), 
//		ゲームパッドコントローラー
///////////////////////////////////////////////////////////////////////////////

'use strict';

( function( Smap ) {

	Smap.controller = ( Smap.controller ) ? Smap.controller : {};

//
//	関数: Smap.controller.gamepad()
//		ゲームパッドコントローラーの生成
//
	Smap.controller.gamepad = function() {
		var
			gamepadController = { 
				id: 'gamepad',
				active: true,
				rollSpeed: 0.6,		// 回転速度（1秒あたりの回転ラジアン角）
				rollAxisSpeed: 0.6,	// 回転速度スティック用（1秒当たりの回転ラジアン角）
				upSpeed: 1.4,		// 上昇(下降)速度（一秒当たりのズームレベル）
				moveSpeed: 200,     // 移動速度（1秒当たりのピクセル数）
				moveAxisSpeed: 600, // 移動速度スティック用（1秒当たりのピクセル数）
			};

		gamepadController.check = function(){
			var
				gamepads = getGamepads(),
				map = this.map;

			if ( this.active && gamepads ) {
				for ( var i = 0; i < gamepads.length; i++ ) {
		        // Chromeのgamepadsではforeachは使えない
					if ( gamepads[i] ) {
				        var
				        	pad = gamepads[i],
				        	refresh = false;
			            if ( pad.buttons[0] && pad.buttons[0].pressed ) { // A Button
				            map.up (  this.upSpeed / 60 );
			            }
			            if ( pad.buttons[3] && pad.buttons[3].pressed ) { // Y Button
			            	map.up ( -this.upSpeed / 60 );
			            }
			            if ( pad.buttons[4] && pad.buttons[4].pressed ) {
			            	map.roll(  this.rollSpeed / 60, 0 );
			            }
			            if ( pad.buttons[5] && pad.buttons[5].pressed ) {
			           		map.roll( -this.rollSpeed / 60, 0 );
			            }
			            if ( pad.buttons[6] && pad.buttons[6].pressed ) {
			            	map.roll(  3 * this.rollSpeed / 60, 0 );
			            }
			            if ( pad.buttons[75] && pad.buttons[7].pressed ) {
			            	map.roll( -3 * this.rollSpeed / 60, 0 );
			            }
			            if ( pad.buttons[12] && pad.buttons[12].pressed ) {
			            	map.move( 0, -this.moveSpeed / 60 );
			            }
			            if ( pad.buttons[13] && pad.buttons[13].pressed ) {
			            	map.move( 0, this.moveSpeed / 60 );
			            }
			            if ( pad.buttons[14] && pad.buttons[14].pressed ) {
			            	map.move( -this.moveSpeed / 60, 0 );
			            }
			            if ( pad.buttons[15] && pad.buttons[15].pressed ) {
			            	map.move(  this.moveSpeed / 60, 0 );
			            }
			            if ( Math.abs( pad.axes[0] ) > 0.01 ) {  // 左スティック左右
			            	map.move( pad.axes[0] * this.moveAxisSpeed / 60, 0 );
			            };
			            if ( Math.abs( pad.axes[1] ) > 0.01 ) {  // 左スティック前後
			          		map.move( 0, pad.axes[1] * this.moveAxisSpeed / 60 );
			            };
			            if (Math.abs(pad.axes[3]) > 0.01) {  //右スティック前後
			         		map.roll( 0, pad.axes[3] * this.rollAxisSpeed / 60 );
			            }
					}
				}
			}
		}
		return gamepadController;
	}

	function getGamepads () {
		var gamepads = null;
		if ( navigator.getGamepads ) {
			gamepads = navigator.getGamepads();
		} else if ( navigator.webkitGetGamepads ) {
			gamepads = navigator.webkitGetGamepads();
		}
		return gamepads;
	}

} ) ( Smap );
