///////////////////////////////////////////////////////////////////////////////
//	モジュール : Smap.controllers.leap.js, 2015-04-26, 西岡 芳晴 ( NISHIOKA Yoshiharu ), 
//		Leap Motionコントローラー
///////////////////////////////////////////////////////////////////////////////

'use strict';

( function( Smap ) {

	Smap.controller = ( Smap.controller ) ? Smap.controller : {};

//
//	関数: Smap.controller.leap()
//		Leap Motionコントローラの生成
//
	Smap.controller.leap = function() {
		var
			leapController = { 
				id: 'leap',
				active: true,
				upF: 0.04,			// 上昇(下降)速度係数（大きいほどほど早くなる）
				moveF: 0.04,       // 移動速度係数（大きいほどほど早くなる）
				rollF: 0.03,		// 回転速度係数（大きいほどほど早くなる）
			},
			hand,
			two = null,
			controller = new Leap.Controller( { enableGestures: true } );

		Leap.loop( function( frame ){
			hand = null;
			if ( frame.hands[0] ) {
				hand = frame.hands[0];
			}
		} );
		controller.connect();

		leapController.check = function(){
			if ( hand ) {
				if ( hand.fingers.length > 3 ) {
					var
						dx = hand.palmPosition[0] * this.moveF,
						dy = hand.palmPosition[2] * this.moveF,
						dz = ( hand.palmPosition[1] - 200 ) * this.upF,
						rx = hand.palmNormal[2] * this.rollF,
						rz = hand.palmNormal[0] * this.rollF;
					this.map.move( dx, dy, dz );
					this.map.up( dz );
//					this.map.move( dx, dy, dz );
					this.map.rollCamera( rx, rz );
					two = null;
				} else if ( hand.fingers.length == 2 ) {
					if ( two ) { // 指二本状態の時
						if ( new Date() - two > 1000) { // 1秒たったら
							this.map.reset();
						}
					} else { // 指二本状態の開始
						two = new Date();
					}
				}
			}
		}
		return leapController;
	}

} ) ( Smap );
