///////////////////////////////////////////////////////////////////////////////
//	Smap.cesium.js, 2015-02-07, 西岡 芳晴 ( NISHIOKA Yoshiharu ), 
//		GSJ.engine.cesium()関数
//		Cesiumラッパー
///////////////////////////////////////////////////////////////////////////////

'use strict';

( function( GSJ ) {

	// Cesium用map生成ファクトリー関数
	GSJ.engine.cesium = function( map ){ 
		var
			base = map.bases[0],
			scene;

		mm = map;
		map.fov = 110;  // 画面上で大体同じになるように設定
		map.original = new Cesium.CesiumWidget( map.owner, {
			terrainProvider : new Cesium.SeamlessTerrainProvider(),
		} ),
		scene = map.original.scene,
		scene.screenSpaceCameraController._minimumZoomRate = 10000;
			// ズームしたときの，ホイールに対する動作制御
		scene.screenSpaceCameraController.minimumZoomDistance = 5000;
			// めり込みにくくするためズーム制限 
			// (ScreenSpaceCameraController.jsでこの数値*0.25以上に高さの制限が掛かる)
		if ( map.center ) {
			var
				c = map.center,
				height = GSJ.zoomToScale( map.zoom, c.lat ) * map.getRangePx();
			scene.animations.add( Cesium.CameraFlightPath.createAnimationCartographic( scene, {
				destination: Cesium.Cartographic.fromDegrees( c.lng, c.lat, height ),
				duration: 0,
			} ) );
		}

		map._layer = function( aLayer ){
			var
				 url = aLayer.url;

			for ( var key in aLayer ) {
				url = url.replace( '{' + key + '}', aLayer[ key ] );
			};
			aLayer.original = map.original.centralBody.imageryLayers.addImageryProvider(
				new Cesium.OpenStreetMapImageryProvider( {
					maximumLevel: aLayer.maxZoom,
					proxy: { getURL: function( aUrl ){
						var
							s = aUrl.split( '/' ),
							p = s.pop().split( '.' ),
							y = p.shift(),
							x = s.pop(),
							z = s.pop();
						return aLayer.getTileUrl( { x: x, y: y, z: z } );
					} },
					url: url.substr( 0, url.indexOf( '{z}' ) ),
					credit: new Cesium.Credit( base.attribution, '', 
						base.attributionUrl ),
				} )
			);
			aLayer.original.alpha = aLayer.opacity;
			return aLayer;
		}

		map._addOverlay = function( aLayer ) {
			aLayer._redraw = function() {
				// 未実装，あえて必要ない？
		//		this.original.redraw();
			}
			aLayer._setOpacity = function( aOpacity ){
				if ( typeof aOpacity !== 'undefined' ) {
					this.original.alpha = aOpacity;
				}
				return this.original.alpha;
			}
			aLayer._setVisibility = function( aVisibility ){
				if ( typeof aVisibility !== 'undefined' ) {
					this.original.show = aVisibility;
				}
				return this.original.show;
			}
			return aLayer;
		};
		map._setCenter = function( aCenter ) {
			if ( typeof aCenter !== 'undefined' ) {
				console.log( aCenter );
				scene.animations.add( Cesium.CameraFlightPath.createAnimationCartographic( scene, {
					destination: Cesium.Cartographic.fromDegrees( aCenter.lng, aCenter.lat, 400000), duration:3000,
				} ) );e
			}
		//	return map.original.getCenter();
			return;
		}
		map._setScale = function( aScale ) {
			ss = map.original.scene;
/*			var
				la = this.original.getView().copyAsLookAt( this.original.ALTITUDE_RELATIVE_TO_GROUND );
			if ( typeof aScale !== 'undefined' ) {
				la.setRange( aScale * this.getRangePx() );
				this.original.getView().setAbstractView( la );
			} else {
				aScale= la.getRange() / map.getRangePx();
			}
*/
			return aScale;
		}
		return map;
	}

} )( GSJMap );
